%%% Simulations for:
%%% Qu, Feng & Damian (submitted), Form similarity-based 
%%% interference effects in word production arise from 
%%% competitive incremental learning
%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clear all; clc;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%% parameter definitions %%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

step=100;                  % number of simulations per trial
Fmax=1;
Fmin=0;
threshold=0.6;                           % absolute threshold
rest=0; 
decay=0.001;                   % decay rate of node activation 
learn=0.4;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%% model simulation %%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Homogeneous context
% t1 represents the steps required to make a response 
% columns represent the four cycles
% rows represent 24 participants * 4 items

t1=zeros(96,4);
for s = 1:24 % 24 participants
    % connection weights from lexical to sublexical nodes
    WWS=0.1*[1 0 1 0 0
        0 1 1 0 0 
        0 0 1 1 0
        0 0 1 0 1];
    
    % connection weights from sublexical to lexical nodes    
    WSW=0.02*[1 0 0 0 
        0 1 0 0  
        1 1 1 1 
        0 0 1 0 
        0 0 0 1];

    for c = 1:4   % four cycles
        for n = 1:4 % four items
            % define the input and nodes initial activation
            input=zeros(1,4);
            input(n)=0.01+max(normrnd(0,0.005),0);
            words=zeros(step,4);
            seg=zeros(step,5); 
            % index for the non-target words
            wind=1:4; % index of words
            wind(n)=[]; % 
            wind_nontarget=wind; % non-target words
            % index for the non-target segments
            sind=1:5;
            if n==1||n==2
                sind([3,n])=[];
                sind_target=[n,3];
            else
                sind([3,n+1])=[];
                sind_target=[3,n+1];
            end
            sind_nontarget=sind;
            dseg = zeros(1,5);
            dseg(sind_target)=1;
            for i = 1:step                
                % input 
                words(i+1,:)=input+seg(i,:)*WSW;
                seg(i+1,:)=words(i,:)*WWS;
                % update activation according to the rule of McClelland and Rumelhart (1981) 
                seg(i+1,:)=seg(i,:)...
                +(seg(i+1,:)>0).*((Fmax-seg(i,:)).*seg(i+1,:)-decay.*(seg(i,:)-rest))...
                +(seg(i+1,:)<=0).*((seg(i,:)-Fmin).*seg(i+1,:)-decay.*(seg(i,:)-rest));
                words(i+1,:)=words(i,:)...
                +(words(i+1,:)>0).*((Fmax-words(i,:)).*words(i+1,:)-decay.*(words(i,:)-rest))...
                +(words(i+1,:)<=0).*((words(i,:)-Fmin).*words(i+1,:)-decay.*(words(i,:)-rest));   
                
                seg(i+1,:)=max(seg(i+1,:),0); % not smaller than 0
                words(i+1,:)=max(words(i+1,:),0);
                seg(i+1,:)=min(seg(i+1,:),1); % not larger than 1
                words(i+1,:)=min(words(i+1,:),1);

                % if the activation of both segment nodes reach the threshold, a response will be made
                if seg(i+1,sind_target)>threshold
                    t1(n+(s-1)*4,c) = i+1; % the 'response time' 
                    % update the connection weight from the word to its corresponding syllable nodes
                    WWS = WWS+learn*(words(i+1,:).')*(seg(i+1,:).*(1-seg(i+1,:)).*(dseg-seg(i+1,:)))
                    break %naming successfully
                end 
            end  
        end
    end
end



% Heterogeneous context
% t2 represents the steps required to make a response 
% columns represent the four cycles
% rows represent 24 participants * 4 items

t2=zeros(96,4);
for s = 1:24
    % connection weights from lexical to sublexical nodes
    WWS=0.1*[1 1 0 0 0 0 0 0 
    0 0 1 1 0 0 0 0
    0 0 0 0 1 1 0 0
    0 0 0 0 0 0 1 1];
    % connetion weight from sublexical to lexical nodes  
    WSW=0.02*[1 0 0 0 
        1 0 0 0  
        0 1 0 0 
        0 1 0 0 
        0 0 1 0
        0 0 1 0
        0 0 0 1
        0 0 0 1];
    for c = 1:4
        for n = 1:4
            input=zeros(1,4);
            input(n)=0.01+max([normrnd(0,0.005),0]);
            words=zeros(step,4);
            seg=zeros(step,8);             
            % index for the non-target words
            wind=1:4; % index of words
            wind(n)=[]; % 
            wind_nontarget=wind; % non-target words
            % index for the non-target segments
            sind=1:8;
            sind_target=[2*n-1,2*n];
            sind([2*n-1,2*n])=[];
            sind_nontarget=sind;
            dseg = zeros(1,8);
            dseg(sind_target)=1;
            for i = 1:step
                words(i+1,:)=input+seg(i,:)*WSW;
                seg(i+1,:)=words(i,:)*WWS;
                seg(i+1,:)=seg(i,:)...
                +(seg(i+1,:)>0).*((Fmax-seg(i,:)).*seg(i+1,:)-decay.*(seg(i,:)-rest))...
                +(seg(i+1,:)<=0).*((seg(i,:)-Fmin).*seg(i+1,:)-decay.*(seg(i,:)-rest));
                words(i+1,:)=words(i,:)...
                +(words(i+1,:)>0).*((Fmax-words(i,:)).*words(i+1,:)-decay.*(words(i,:)-rest))...
                +(words(i+1,:)<=0).*((words(i,:)-Fmin).*words(i+1,:)-decay.*(words(i,:)-rest));  
                seg(i+1,:)=max(seg(i+1,:),0); % not smaller than 0
                words(i+1,:)=max(words(i+1,:),0);
                seg(i+1,:)=min(seg(i+1,:),1); % not larger than 1
                words(i+1,:)=min(words(i+1,:),1);
                % if the activation of both segment nodes reach the threshold, a response will be made
                if seg(i+1,2*(n-1)+1)>threshold && seg(i+1,2*(n-1)+2)>threshold
                    t2(n+(s-1)*4,c)=i+1;
                    % update the connection weight from the word to its corresponding syllable nodes
                    WWS = WWS+learn*(words(i+1,:).')*(seg(i+1,:).*(1-seg(i+1,:)).*(dseg-seg(i+1,:)));
                    break %naming successfully
                end 
            end  
        end
    end
end

mean_t_homo=mean(t1)
mean_t_hete=mean(t2)